Package: GPSeqClus
Type: Package
Title: Sequential Clustering Algorithm for Location Data
Version: 1.5.0
Date: 2026-05-17
Authors@R: c(person(family = "Clapp", role = "aut"),
             person(given = "J.", family = "G.", role = "aut"),
             person(given = "Justin", family = "Clapp", role = "cre", email = "justin.clapp@wyo.gov"))
Maintainer: Justin Clapp <justin.clapp@wyo.gov>
Description: Applies sequential clustering algorithm to animal location data 
    based on user-defined parameters. Plots interactive cluster maps and 
    provides a summary dataframe with attributes for each cluster commonly
    used as covariates in subsequent modeling efforts. Additional functions
    provide individual keyhole markup language plots for quick assessment,
    and export of global positioning system exchange format files for
    navigation purposes. 
    Methods can be found at <doi:10.1111/2041-210X.13572>.
Depends: R (>= 3.5)
Imports: geosphere, htmlwidgets, leaflet, leaflet.extras, plyr, purrr,
        sp, sf, stats, suncalc, tcltk, utils
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-05-17 15:04:20 UTC; Jugclapp
Author: Clapp [aut],
  J. G. [aut],
  Justin Clapp [cre]
Repository: CRAN
Date/Publication: 2026-05-17 22:40:26 UTC
Built: R 4.4.3; ; 2026-05-17 23:58:59 UTC; unix
