# Generated by roxygen2: do not edit by hand

export(GPSeq_clus)
export(exp_clus_gpx)
export(ind_clus_kml)
import(leaflet)
import(plyr)
importFrom(geosphere,distHaversine)
importFrom(geosphere,distm)
importFrom(htmlwidgets,onRender)
importFrom(leaflet.extras,addSearchFeatures)
importFrom(leaflet.extras,searchFeaturesOptions)
importFrom(purrr,keep)
importFrom(purrr,walk)
importFrom(sf,st_as_sf)
importFrom(sf,st_write)
importFrom(sp,CRS)
importFrom(sp,SpatialPointsDataFrame)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(suncalc,getSunlightTimes)
importFrom(tcltk,setTkProgressBar)
importFrom(tcltk,tkProgressBar)
importFrom(utils,globalVariables)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
