#' @title Sample Data for Sequential Clustering Routine
#'
#' @description A \code{dataframe} containing a subset of GPS location data from 2 male and 1 female mountain lions used
#' for testing and running sequential cluster function examples.Example data provided by Wyoming Game and Fish Department, 2020.
#'
#' @format A \code{dataframe} containing 4 columns:
#' \describe{
#' \item{AID}{animal identification}
#' \item{TelemDate}{location timestamp in POSIXct format}
#' \item{Lat}{latitude coordinates}
#' \item{Long}{longitude coordinates}
#' }
#'
"ML_ex_dat"
